from django import template
from panorama.models import PanoramaInfo
from django.conf import settings
register = template.Library()

def panorama_js():
    return """
    <script type="text/javascript" src="%spanorama/js/jquery.js"></script>
    <script type="text/javascript" src="%spanorama/js/jquery.fancybox-1.3.1.pack.js"></script>
    <script type="text/javascript" src="%spanorama/js/jquery.panorama.js"></script>
    """ % ((settings.STATIC_URL,)*3)

panorama_js = register.simple_tag(panorama_js)


def show_panorama(panorama,position=None):
    infos = []
    for subinfo in PanoramaInfo.__subclasses__():
        infos +=subinfo.objects.filter(panoramainfo_ptr__panorama=panorama)
    return {'panorama':panorama,
            'position':position,
            'panoramainfos':infos}

show_panorama = register.inclusion_tag('panorama/img.html')(show_panorama)



from django.contrib import admin
from django import forms
from models import Panorama, PanoramaExternalLink, PanoramaNote
from django.core.urlresolvers import reverse
from multilingual.admin import MultilingualModelAdmin
from multilingual.admin import MultilingualModelAdminForm

class PanoramaWidget(forms.Select):
    class Media:
        js = ('panorama/js/jquery.js','panorama/js/panorama_admin.js',)

    def build_attrs(self, extra_attrs=None, **kwargs):
        attrs = super(PanoramaWidget,self).build_attrs(extra_attrs,**kwargs)
        json_url = reverse('panorama.views.panorama_json',kwargs={'pk':999999})
        attrs['onchange']="update_panorama_widget(this,map,'%s')" % json_url
        return attrs

class PanoramaInfoAdmin(admin.ModelAdmin):
    """
      Base ModelAdmin for models inheriting PanoramaInfo.

      Defines common values such as list_display and list_filter.
    """
    base_list_display = ('title','panorama',)
    base_list_filter = ('panorama',)

    def __init__(self, *args, **kwargs):
        self.list_display = self.base_list_display + self.list_display
        self.list_filter = self.base_list_filter + self.list_filter
        super(PanoramaInfoAdmin,self).__init__(*args, **kwargs)

class PanoramaExternalLinkForm(MultilingualModelAdminForm):
    panorama = forms.ModelChoiceField(queryset=Panorama.objects.all(),widget=PanoramaWidget())
    class Meta:
        model = PanoramaExternalLink

class PanoramaExternalLinkAdmin(MultilingualModelAdmin, PanoramaInfoAdmin):
    form = PanoramaExternalLinkForm
    #TODO: this use_fieldests don't work in django 1.3-beta-1, watch multilingual_ng
    #use_fieldsets = (
    #        (None, {'fields': (('title', 'panorama', 'url'),),}),
    #        ('Advanced options', {
    #            'classes':('collapse',),
    #            'fields':(('x0','x1','y0','y1'),),
    #            }),
    #        )
    list_display = ('url',)

class PanoramaNoteForm(MultilingualModelAdminForm):
    panorama = forms.ModelChoiceField(queryset=Panorama.objects.all(),widget=PanoramaWidget())
    class Meta:
        model = PanoramaNote

class PanoramaNoteAdmin(MultilingualModelAdmin, PanoramaInfoAdmin):
    form = PanoramaNoteForm
    #use_fieldsets = (
    #        (None, {'fields': (('title',), ('description',), ('panorama' ),),}),
    #        ('Advanced options', {
    #            'classes':('collapse',),
    #            'fields':(('x0','x1','y0','y1'),),
    #            }),
    #        )


admin.site.register(Panorama, MultilingualModelAdmin)
admin.site.register(PanoramaExternalLink, PanoramaExternalLinkAdmin)
admin.site.register(PanoramaNote, PanoramaNoteAdmin)



from django.db import models
from multilingual.translation import TranslationModel
from django.utils.translation import ugettext_lazy as _
from tinymce.models import HTMLField

# Create your models here.

class Panorama (models.Model):
    """
      This model stores a panoramic image.
    """
    class Translation(TranslationModel):
        title = models.CharField(name=_('title'), max_length=30)
    image = models.ImageField(name=_('image'), upload_to='panoramas/')

    class Meta:
        verbose_name = _('panorama')
        verbose_name_plural = _('panoramas')

    def __unicode__(self):
        return self.title or 'New Panorama'

    @models.permalink
    def get_absolute_url(self):
        return ('panorama_detail', [str(self.id)])


class PanoramaInfo(models.Model):
    """
      This base model defines the attributes needed to attach info to a
      panorama. Attached info will be seen as rectangles floating over
      panorama.
      
      Not really useful alone, see PanoramaNote, PanoramaExternalLink, etc.
    """
    panorama = models.ForeignKey('Panorama', name=_('panorama'))
    x0 = models.IntegerField(name=u'X0')
    x1 = models.IntegerField(name=u'X1')
    y0 = models.IntegerField(name=u'Y0')
    y1 = models.IntegerField(name=u'Y1')
    type = None
    url = None
    alt = None

    #class Meta:
    #    abstract = True


class PanoramaExternalLink(PanoramaInfo):
    """
      Show a external web inside a iframe when clicked in this
      PanoramaInfo.
    """
    class Translation(TranslationModel):
        title = models.CharField(name=_('title'), max_length=30)
    url = models.CharField(name=u'URL', max_length=200)
    type = 'iframe'
    def __unicode__(self):
        return self.title or 'New Panorama External Link'

    def alt(self):
        return self.title

class PanoramaNote(PanoramaInfo):
    """
      Show a HTML note as a popup when clicked this PanoramaInfo.
    """
    class Translation(TranslationModel):
        title = models.CharField(name=_('title'), max_length=30)
        description = HTMLField(name=_('description'), blank=True)
    type = 'ajax'

    @models.permalink
    def get_absolute_url(self):
        return ('panorama.views.show_note', [str(self.id)])

    def __unicode__(self):
        return self.title or 'New Panorama Note'

    def alt(self):
        return self.title

    def url(self):
        return self.get_absolute_url()



from django.test import TestCase
from panorama.models import *

class AppTestCase(TestCase):
    """
    Populate this class with unit tests for your application
    """

    fixtures = ('panorama',)
    
    urls = 'panorama.test_urls'
    
    def testTemplatetagShowPanoramaLocateInfos(self):
        "Test the show_panorama templatetag: valid panoramainfos"
        from panorama.templatetags.panorama import show_panorama
        panorama = Panorama.objects.get(pk=1) #Get the 'Hall' panorama
        note = PanoramaNote.objects.get(pk=1)
        note2 = PanoramaNote.objects.get(pk=3)
        link = PanoramaExternalLink.objects.get(pk=2)
        context = show_panorama(panorama)
        panoramainfos = context['panoramainfos']
        self.assertEqual(len(panoramainfos),2)
        self.assertTrue(note in panoramainfos)
        self.assertTrue(link in panoramainfos)
        self.assertFalse(note2 in panoramainfos)

    def testPanoramaDetail(self):
        "Test the detail view of a panorama"
        response = self.client.get('/panoramas/1')
        self.assertEqual(response.status_code, 200)
        #Verify jquery.advancedpanorama is loaded
        self.assertContains(response,'panorama/js/jquery.panorama.js')
        #Verify jquery.advancedpanorama is launched for this panorama
        self.assertContains(response,'$("img.advancedpanorama").panorama({')
        #Verify two panoramainfos are shown
        self.assertContains(response,'<area ',2)
        #Verify one panoramainfos is iframe (externallink)
        self.assertContains(response,'class="iframe"',1)
        #Verify one panoramainfos is ajax (note)
        self.assertContains(response,'class="ajax"',1)

    def testPanoramaListing(self):
        "Test the listing view of panoramas"
        response = self.client.get('/panoramas/')
        self.assertEqual(response.status_code, 200)
        #Verify three panoramas are shown
        self.assertContains(response,'<a href="/panoramas/',3)
